package me.mrletsplay.scraperbot;

import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;

import me.mrletsplay.mrcore.io.IOUtils;
import me.mrletsplay.mrcore.json.JSONObject;
import me.mrletsplay.mrcore.json.converter.JSONConverter;
import me.mrletsplay.mrcore.json.converter.SerializationOption;
import me.mrletsplay.scraperbot.command.CommandScrape;
import me.mrletsplay.scraperbot.command.SimpleCommand;
import net.dv8tion.jda.api.JDA;
import net.dv8tion.jda.api.JDABuilder;
import net.dv8tion.jda.api.OnlineStatus;
import net.dv8tion.jda.api.entities.Activity;
import net.dv8tion.jda.api.events.interaction.command.CommandAutoCompleteInteractionEvent;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.hooks.ListenerAdapter;
import net.dv8tion.jda.api.interactions.commands.build.Commands;
import net.dv8tion.jda.api.interactions.commands.build.SlashCommandData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandData;
import net.dv8tion.jda.api.interactions.commands.build.SubcommandGroupData;
import net.dv8tion.jda.api.requests.restaction.CommandListUpdateAction;

public class ScraperBot extends ListenerAdapter {

	public static final Path CONFIG_PATH = Paths.get("config.json");

	public static final ScheduledExecutorService EXECUTOR = Executors.newScheduledThreadPool(0);
	public static final List<ScraperTask> TASKS = new ArrayList<>();

	public static final List<SimpleCommand> COMMANDS = Arrays.asList(
		new CommandScrape()
	);

	public static void main(String[] args) throws Exception {
		if(!Files.exists(CONFIG_PATH)) {
			IOUtils.createFile(CONFIG_PATH.toFile());
			Files.writeString(CONFIG_PATH, ScraperBotConfig.createDefault().toJSON(SerializationOption.DONT_INCLUDE_CLASS).toFancyString(), StandardCharsets.UTF_8);
			return;
		}

		String cfgString = Files.readString(Paths.get("config.json"));
		ScraperBotConfig config = JSONConverter.decodeObject(new JSONObject(cfgString), ScraperBotConfig.class);

		JDA jda = JDABuilder.createDefault(config.getToken())
			.setStatus(OnlineStatus.IDLE)
			.setActivity(Activity.watching("you and all of your websites \uD83D\uDD0D"))
			.addEventListeners(new ScraperBot())
			.setAutoReconnect(true)
			.build();

		jda.awaitReady();

		CommandListUpdateAction a;
		if(config.isUseTestingServer()) {
			System.out.println("Using testing server: " + config.getTestingServer());
			a = jda.getGuildById(config.getTestingServer()).updateCommands();
		}else {
			a = jda.updateCommands();
		}

		for(SimpleCommand command : COMMANDS) {
			SlashCommandData d = Commands.slash(command.getName(), command.getDescription());

			if(!command.getChildren().isEmpty()) {
				for(SimpleCommand subCommand : command.getChildren()) {
					if(!subCommand.getChildren().isEmpty()) {
						SubcommandGroupData gd = new SubcommandGroupData(subCommand.getName(), subCommand.getDescription());

						for(SimpleCommand groupCommand : subCommand.getChildren()) {
							gd.addSubcommands(new SubcommandData(groupCommand.getName(), groupCommand.getDescription()).addOptions(groupCommand.getOptions()));
						}

						d.addSubcommandGroups(gd);
					}else {
						d.addSubcommands(new SubcommandData(subCommand.getName(), subCommand.getDescription()).addOptions(subCommand.getOptions()));
					}
				}
			}else {
				d.addOptions(command.getOptions());
			}

			a.addCommands(d);
		}

		a.queue();
	}

	@Override
	public void onSlashCommandInteraction(SlashCommandInteractionEvent event) {
		List<String> parts = new ArrayList<>(Arrays.asList(event.getFullCommandName().split(" ")));

		String c = parts.remove(0);
		SimpleCommand cmd = COMMANDS.stream().filter(cm -> cm.getName().equals(c)).findFirst().orElse(null);
		if(cmd == null) return;

		while(!parts.isEmpty()) {
			String sc = parts.remove(0);
			cmd = cmd.getChildren().stream().filter(ch -> ch.getName().equals(sc)).findFirst().orElse(null);
			if(cmd == null) return;
		}

		cmd.action(event);
	}

	@Override
	public void onCommandAutoCompleteInteraction(CommandAutoCompleteInteractionEvent event) {
		List<String> parts = new ArrayList<>(Arrays.asList(event.getFullCommandName().split(" ")));

		String c = parts.remove(0);
		SimpleCommand cmd = COMMANDS.stream().filter(cm -> cm.getName().equals(c)).findFirst().orElse(null);
		if(cmd == null) return;

		while(!parts.isEmpty()) {
			String sc = parts.remove(0);
			cmd = cmd.getChildren().stream().filter(ch -> ch.getName().equals(sc)).findFirst().orElse(null);
			if(cmd == null) return;
		}

		cmd.complete(event);
	}

}
