package me.mrletsplay.scraperbot.command;

import java.util.List;
import java.util.stream.Collectors;

import me.mrletsplay.scraperbot.FilterList;
import me.mrletsplay.scraperbot.ScraperBot;
import me.mrletsplay.scraperbot.ScraperTask;
import me.mrletsplay.scraperbot.ScraperTask.Strategy;
import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.utils.messages.MessageCreateBuilder;

public class CommandScrape extends SimpleParentCommand {

	public CommandScrape() {
		super("scrape", "Scrape a website");

		OptionData strategy = new OptionData(OptionType.STRING, "strategy", "Scraping strategy", false);
		for(Strategy s : Strategy.values()) {
			strategy.addChoice(s.getFriendlyName(), s.name());
		}

		addChild("create", "Create a scraping task", event -> {
			ScraperTask task = new ScraperTask(event.getOption("first_url").getAsString());

			OptionMapping map = event.getOption("strategy");
			if(map != null) {
				task.setStrategy(Strategy.valueOf(map.getAsString()));
			}

			ScraperBot.TASKS.add(task);

			event.reply(String.format("Task created: `%s`", task.getID())).queue();
		},
			new OptionData(OptionType.STRING, "first_url", "Initial URL to scrape", true),
			strategy);

		addChild("info", "Show info about a scraping task", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			EmbedBuilder b = new EmbedBuilder();
			b.setTitle(task.getID());

			b.addField("URL", task.getFirstURL(), true);
			b.addField("Strategy", task.getStrategy().getFriendlyName(), true);
			b.addField("Started", task.getStartTime() == 0 ? "No" : (System.currentTimeMillis() - task.getStartTime()) / (1000 * 60) + " minute(s) ago", true);

			b.addField("Include: Exact", task.getInclude().getExact().stream().collect(Collectors.joining("\n")), true);
			b.addField("Include: Prefix", task.getInclude().getPrefix().stream().collect(Collectors.joining("\n")), true);
			b.addField("Include: Regex", task.getInclude().getRegex().stream().collect(Collectors.joining("\n")), true);

			b.addField("Exclude: Exact", task.getExclude().getExact().stream().collect(Collectors.joining("\n")), true);
			b.addField("Exclude: Prefix", task.getExclude().getPrefix().stream().collect(Collectors.joining("\n")), true);
			b.addField("Exclude: Regex", task.getExclude().getRegex().stream().collect(Collectors.joining("\n")), true);

			b.addField("Find: Exact", task.getFind().getExact().stream().collect(Collectors.joining("\n")), true);
			b.addField("Find: Prefix", task.getFind().getPrefix().stream().collect(Collectors.joining("\n")), true);
			b.addField("Find: Regex", task.getFind().getRegex().stream().collect(Collectors.joining("\n")), true);

			event.reply(new MessageCreateBuilder().addEmbeds(b.build()).build()).queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		}, new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true));

		addChild("start", "Start a scraping task", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			task.start(event.getChannel());
			event.reply("Task started!").queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		}, new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true));

		addChild("cancel", "Cancel a scraping task", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			task.cancel();
			event.reply("Task cancelled!").queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		}, new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true));

		addChild("cookie", "Yum, cookies!", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			task.setCookie(event.getOption("cookie").getAsString());
			event.reply("Cookie set! :cookie: :yum:").queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		},
			new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true),
			new OptionData(OptionType.STRING, "cookie", "Cookie to set", true));

		addChild("useragent", "Set the UA for requests", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			task.setUserAgent(event.getOption("user_agent").getAsString());
			event.reply("UA set!").queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		},
			new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true),
			new OptionData(OptionType.STRING, "user_agent", "UA to set", true));

		SimpleParentCommand cmd = new SimpleParentCommand("filter", "Manage filters");
		addChild(cmd);

		OptionData filter = new OptionData(OptionType.STRING, "filter", "Filter to update", true)
			.addChoice("Include", "include")
			.addChoice("Exclude", "exclude")
			.addChoice("Find", "find");
		OptionData type = new OptionData(OptionType.STRING, "type", "Type of filter", true)
			.addChoice("Exact Match", "exact")
			.addChoice("Has Prefix", "prefix")
			.addChoice("Regex", "regex");

		cmd.addChild("add", "Add a value to a filter", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			String filterS = event.getOption("filter").getAsString();
			String typeS = event.getOption("type").getAsString();
			List<String> vals = getFilterVals(task, filterS, typeS);
			vals.add(event.getOption("value").getAsString());
			event.reply("Filter updated").queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		},
			new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true),
			filter,
			type,
			new OptionData(OptionType.STRING, "value", "Value", true));

		cmd.addChild("remove", "Remove a value from a filter", event -> {
			String id = event.getOption("id").getAsString();
			ScraperTask task = ScraperBot.TASKS.stream().filter(t -> t.getID().equals(id)).findFirst().orElse(null);
			if(task == null) {
				event.reply("Task not found").queue();
				return;
			}

			String filterS = event.getOption("filter").getAsString();
			String typeS = event.getOption("type").getAsString();
			List<String> vals = getFilterVals(task, filterS, typeS);
			vals.remove(event.getOption("value").getAsString());
			event.reply("Filter updated").queue();
		}, event -> {
			if(event.getFocusedOption().getName().equals("id")) {
				event.replyChoiceStrings(ScraperBot.TASKS.stream().map(t -> t.getID()).toList()).queue();
			}
		},
			new OptionData(OptionType.STRING, "id", "Task ID", true).setAutoComplete(true),
			filter,
			type,
			new OptionData(OptionType.STRING, "value", "Value", true));
	}

	private static List<String> getFilterVals(ScraperTask task, String filterS, String typeS) {
		FilterList list;
		switch(filterS) {
			case "include":
			default:
				list = task.getInclude();
				break;
			case "exclude":
				list = task.getExclude();
				break;
			case "find":
				list = task.getFind();
				break;
		}

		List<String> vals;
		switch(typeS) {
			case "exact":
			default:
				vals = list.getExact();
				break;
			case "prefix":
				vals = list.getPrefix();
				break;
			case "regex":
				vals = list.getRegex();
				break;
		}

		return vals;
	}

}
