package me.mrletsplay.scraperbot.command;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Consumer;

import net.dv8tion.jda.api.events.interaction.command.CommandAutoCompleteInteractionEvent;
import net.dv8tion.jda.api.events.interaction.command.SlashCommandInteractionEvent;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;

public abstract class SimpleCommand {

	private String name, description;
	private List<SimpleCommand> children;

	public SimpleCommand(String name, String description) {
		this.name = name;
		this.description = description;
		this.children = new ArrayList<>();
	}

	public String getName() {
		return name;
	}

	public String getDescription() {
		return description;
	}

	public void addChild(SimpleCommand cmd) {
		children.add(cmd);
	}

	public void addChild(String name, String description, Consumer<SlashCommandInteractionEvent> action, Consumer<CommandAutoCompleteInteractionEvent> complete, OptionData... options) {
		List<OptionData> opts = Arrays.asList(options);
		addChild(new SimpleCommand(name, description) {

			@Override
			public void action(SlashCommandInteractionEvent event) {
				action.accept(event);
			}

			@Override
			public void complete(CommandAutoCompleteInteractionEvent event) {
				if(complete != null) complete.accept(event);
			}

			@Override
			public List<OptionData> getOptions() {
				return opts;
			}

		});
	}

	public void addChild(String name, String description, Consumer<SlashCommandInteractionEvent> action, OptionData... options) {
		addChild(name, description, action, null, options);
	}

	public List<SimpleCommand> getChildren() {
		return children;
	}

	public abstract void action(SlashCommandInteractionEvent event);

	public void complete(CommandAutoCompleteInteractionEvent event) {}

	public abstract List<OptionData> getOptions();

}
